'use strict';

var flightModules = require('react-noop-renderer/flight-modules');
var ReactFlightServer = require('react-server/flight');

/**
 * This is a renderer of React that doesn't have a render target output.
 * It is useful to demonstrate the internals of the reconciler in isolation
 * and for testing semantics of reconciliation separate from the host
 * environment.
 */
const ReactNoopFlightServer = ReactFlightServer({
  scheduleWork(callback) {
    callback();
  },

  beginWriting(destination) {},

  writeChunk(destination, chunk) {
    destination.push(chunk);
  },

  writeChunkAndReturn(destination, chunk) {
    destination.push(chunk);
    return true;
  },

  completeWriting(destination) {},

  close(destination) {},

  closeWithError(destination, error) {},

  flushBuffered(destination) {},

  stringToChunk(content) {
    return content;
  },

  stringToPrecomputedChunk(content) {
    return content;
  },

  isModuleReference(reference) {
    return reference.$$typeof === Symbol.for('react.module.reference');
  },

  getModuleKey(reference) {
    return reference;
  },

  resolveModuleMetaData(config, reference) {
    return flightModules.saveModule(reference.value);
  }

});

function render(model, options) {
  const destination = [];
  const bundlerConfig = undefined;
  const request = ReactNoopFlightServer.createRequest(model, bundlerConfig, options ? options.onError : undefined, options ? options.context : undefined, options ? options.identifierPrefix : undefined);
  ReactNoopFlightServer.startWork(request);
  ReactNoopFlightServer.startFlowing(request, destination);
  return destination;
}

exports.render = render;
//# sourceMappingURL=react-noop-renderer-flight-server.production.min.js.map
